@@ -84,10 +84,12 @@ class TMC_2209:
         self.log("Init", Loglevel.info.value)
         GPIO.setwarnings(False)
         GPIO.setmode(GPIO.BCM)
-        GPIO.setup(self._pin_step, GPIO.OUT)
-        GPIO.setup(self._pin_dir, GPIO.OUT)
-        GPIO.setup(self._pin_en, GPIO.OUT)
-        GPIO.output(self._pin_dir, self._direction)
+        self.log("STEP Pin: " + str(self._pin_step), Loglevel.debug.value)
+        self.log("DIR Pin: " + str(self._pin_dir), Loglevel.debug.value)
+        self.log("EN Pin: " + str(self._pin_en), Loglevel.debug.value)
+        GPIO.setup(self._pin_step, GPIO.OUT, initial=GPIO.LOW)
+        GPIO.setup(self._pin_dir, GPIO.OUT, initial=self._direction)
+        GPIO.setup(self._pin_en, GPIO.OUT, initial=GPIO.HIGH)
         self.log("GPIO Init finished", Loglevel.info.value)
         if(not no_uart):
             self.readStepsPerRevolution()
@@ -105,8 +107,9 @@ class TMC_2209:
         try:
             self.setMotorEnabled(False)
             GPIO.cleanup()
+            self.log("GPIO cleanup")
         except:
-            self.log("already cleaned up")
+            self.log("GPIO already cleaned up")
 
 
 #-----------------------------------------------------------------------
@@ -530,23 +533,33 @@ class TMC_2209:
 # run_current in mA
 # check whether Vref is actually 1.2V
 #-----------------------------------------------------------------------
-    def setCurrent(self, run_current, hold_current_multiplier = 0.5, hold_current_delay = 10, Vref = 1.2):
+    def setCurrent(self, run_current, hold_current_multiplier = 0.5, hold_current_delay = 10, use_Vref=False, Vref = 1.2):
         CS_IRun = 0
         Rsense = 0.11
         Vfs = 0
 
-        if(self.getVSense()):
-            self.log("Vsense: 1", Loglevel.info.value)
-            Vfs = 0.180 * Vref / 2.5
+        if (use_Vref):
+            self.setIScaleAnalog(True)
         else:
-            self.log("Vsense: 0", Loglevel.info.value)
-            Vfs = 0.325 * Vref / 2.5
-            
+            Vref=2.5
+            self.setIScaleAnalog(False)
+
+        Vfs = 0.325 * Vref / 2.5
         CS_IRun = 32.0*1.41421*run_current/1000.0*(Rsense+0.02)/Vfs - 1
 
+        # If Current Scale is too low, turn on high sensitivity VSsense and calculate again
+        if(CS_IRun < 16):
+            self.log("CS too low; switching to VSense True", Loglevel.info.value)
+            Vfs = 0.180 * Vref / 2.5
+            CS_IRun = 32.0*1.41421*run_current/1000.0*(Rsense+0.02)/Vfs - 1
+            self.setVSense(True)
+        else: # If CS >= 16, turn off high_senser
+            self.log("CS in range; using VSense False", Loglevel.info.value)
+            self.setVSense(False)
+
         CS_IRun = min(CS_IRun, 31)
         CS_IRun = max(CS_IRun, 0)
-        
+
         CS_IHold = hold_current_multiplier * CS_IRun
 
         CS_IRun = round(CS_IRun)
@@ -557,6 +570,10 @@ class TMC_2209:
         self.log("CS_IHold: " + str(CS_IHold), Loglevel.info.value)
         self.log("Delay: " + str(hold_current_delay), Loglevel.info.value)
 
+        # return (float)(CS+1)/32.0 * (vsense() ? 0.180 : 0.325)/(Rsense+0.02) / 1.41421 * 1000;
+        run_current_actual = (CS_IRun+1)/32.0 * (Vfs)/(Rsense+0.02) / 1.41421 * 1000
+        self.log("actual current: "+str(round(run_current_actual))+" mA", Loglevel.info.value)
+
         self.setIRun_Ihold(CS_IHold, CS_IRun, hold_current_delay)
 
   
